<?php

namespace Ranig;

use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\Structures\TrackMania\OnWayPointEventStructure;
use ManiaControl\Callbacks\Structures\TrackMania\OnScoresStructure;
use ManiaControl\Commands\CommandListener;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Bills\BillManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerActions;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Statistics\StatisticCollector;
use ManiaControl\Statistics\StatisticManager;
use ManiaControl\Utils\Formatter;
use Maniaplanet\DedicatedServer\Structures\AbstractStructure;

/**
 */
class LotteryPlugin implements Plugin, CallbackListener, CommandListener {
	/*
	 * Constants
	 */
	const PLUGIN_ID                 = 139;
	const PLUGIN_VERSION            = 1;
	const PLUGIN_NAME               = 'Lottery Plugin';
	const PLUGIN_AUTHOR             = 'Ranig';
	const SETTING_MIN_PLAYERS       = 'Minimum number of players that finish a map to activate lottery';
	const SETTING_LOTTERY_VALUE     = 'Number of Planets to be paid by the lottery';
	const SETTING_MIN_PLANETS       = 'Minimum number of Planets on the server to activate payments';
	const SETTING_USE_TOTAL_PLAYERS = 'Use the total number of players instead of number of players that finish the map to activate payment';

	/*
	 * Private properties
	 */
	/** @var ManiaControl $maniaControl * */
	private $maniaControl = null;
	private $paymentDone = null;
	private $podiumActivated = null;
	private $playerFinished = null;

	/**
	 * @see \ManiaControl\Plugins\Plugin::prepare()
	 */
	public static function prepare(ManiaControl $maniaControl) {
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getId()
	 */
	public static function getId() {
		return self::PLUGIN_ID;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getName()
	 */
	public static function getName() {
		return self::PLUGIN_NAME;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getVersion()
	 */
	public static function getVersion() {
		return self::PLUGIN_VERSION;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getAuthor()
	 */
	public static function getAuthor() {
		return self::PLUGIN_AUTHOR;
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::getDescription()
	 */
	public static function getDescription() {
		return "Lottery Plugin, pay Planets to a random finisher on each map";
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::load()
	 */
	public function load(ManiaControl $maniaControl) {
		$this->maniaControl = $maniaControl;

		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_MIN_PLAYERS, 5);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_LOTTERY_VALUE, 15);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_MIN_PLANETS, 5000);
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_USE_TOTAL_PLAYERS, false);

		// Callbacks
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::TM_SCORES, $this, 'handleScores');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_PODIUMSTART, $this, 'handlePodium');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::TM_ONFINISHLINE, $this, 'handleFinishCallback');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::BEGINMAP, $this, 'handleBeginMap');
		
		$this->paymentDone = false;
		$this->podiumActivated = false;
		$this->playerFinished = array();
	}

	/**
	 * @see \ManiaControl\Plugins\Plugin::unload()
	 */
	public function unload() {
	}

	/**
	 * Reinitializes internal buffers at the start of the map
	 */
	public function handleBeginMap() {
		$this->paymentDone = false;
		$this->podiumActivated = false;
		$this->playerFinished = array();
	}

	/**
	 * Adds a player to the list of players who finished
	 */
	public function handleFinishCallback(OnWayPointEventStructure $structure) {
		$this->playerFinished[$structure->getLogin()] = true;
	}

	/**
	 * Pay winners
	 */
	public function handleScores(OnScoresStructure $fullScoreStructure) {
		if ($fullScoreStructure->getSection() != 'EndMap') {
			return;
		}

		if ($this->maniaControl->getClient()->getServerPlanets() < $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MIN_PLANETS)) {
			$this->maniaControl->getChat()->sendInformation('Not enough Planets on the server for lottery!');
			return;
		}
		
		$scores = $fullScoreStructure->getPlayerScores();
		
		$numberPlayers = count($scores);
		$numberFinishers = count($this->playerFinished);

		$index = 0;

		if ($this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_USE_TOTAL_PLAYERS)) {
			if ($numberPlayers < $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MIN_PLAYERS)) {
				$this->maniaControl->getChat()->sendInformation('Not enough players connected for lottery!');
				return;
			}
			
			$index = rand(0, $numberPlayers - 1) + 1;
		} else {
			if ($numberFinishers < $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_MIN_PLAYERS)) {
				$this->maniaControl->getChat()->sendInformation('Not enough players finished for lottery!');
				return;
			}
			
			$index = rand(0, $numberFinishers - 1) + 1;
		}
		
		$announce = 'Unable to find a lottery winner!';
		$payment = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_LOTTERY_VALUE);
		
		foreach ($scores as $player => $score) {
			if ($score->getRank() == $index) {
				$playerStructure = $this->maniaControl->getPlayerManager()->getPlayer($player);
				$announce = $playerStructure->getEscapedNickname();
				$message = 'You won ' . $payment . ' Planets!';
				$this->maniaControl->getBillManager()->sendPlanets(function ($data, $status) use ($player, $payment) {
					switch ($status) {
						case BillManager::PAYED_FROM_SERVER:
							$message = 'You won ' . $payment . ' Planets!';
							$this->maniaControl->getChat()->sendSuccess($message, $player);
							break;
						case BillManager::ERROR_WHILE_TRANSACTION:
							$message = $data;
							$this->maniaControl->getChat()->sendError($message, $player);
							break;
					}
				}, $player, $payment, $message);
				$announce = $announce . ' won the lottery!';
			}
		}
		
		$this->maniaControl->getChat()->sendInformation($announce);
	}
}